const { Nuxt, Builder, Generator } = require('nuxt')
const { KiwiGenerator } = require('./utils/generator')
const router = require('./utils/router')
const express = require('express')
const path = require('path')
const app = express()
const cors = require('cors')
const fs = require('fs-extra')
const del = require('del')
const config = require(path.join(process.cwd(), 'kiwi.config.js'))

// for ENV VARs
// TODO: put PORT into options

module.exports = async (command, cliArgs) => {
  // TODO : error handling
  const jsonPath = cliArgs.jsonPath

  const kiwiPath = path.resolve(process.cwd(), cliArgs.kiwiPath)
  // TODO: find file naming conventions

  const kiwiMapPath = path.join(jsonPath, 'kiwimap.json')
  const kiwiBCPath = path.join(jsonPath, 'basecontents.json')
  const cliPath = __dirname
  const port =  Number(cliArgs.port) || process.env.PORT
  const isDebug = cliArgs.debug
  const configs = {
    kiwiPath, kiwiMapPath, kiwiBCPath, cliPath, port, isDebug
  }

  const isDev = (command == 'dev')
  const isGen = (command == 'generate')

  process.env.DEBUG = 'nuxt:*'

  // for CORS 
  app.use(cors())
  app.use('/static', express.static('static'))
  // insert kiwiOptions in req.  
  let { kiwiMap, baseContents } = await new KiwiGenerator(configs).generate()

  app.use((req, res, next)=>{
    // TODO check atime for .json and cache kiwiMap and baseConents
    Object.assign(req, {
      kiwiConfigs: configs,
      kiwiPayload: {
        kiwiMap, baseContents
      }
    })
    next()
  })

  // This is for Routing
  app.use('/api', router)

  // Set up nuxt.confg.js from kiwi.config.js

  config.generate = { routes: kiwiMap.map(x=>x.route) };
  config.srcDir = path.resolve(__dirname)
  config.rootDir = path.resolve(process.cwd())
  if(!isDev){
    config.dev = false
  }

  const nuxt = new Nuxt(config)
  const builder = new Builder(nuxt)

  if(isDev){
    await builder.build()
  }
  let server = app.listen(port, '0.0.0.0')
  console.log('* server is listening on '+port)
  if(isGen){
    console.log('* start nuxt:generate')
    nuxt.hook('generate:done', async (nuxt, error)=> {
      console.log(error)
      console.log('* nuxt:generate done, all static files are generated in /dist')
      await fs.copy(path.join(process.cwd(), 'dist'), process.cwd())
      console.log('* moved every files in /dist to current working directory')
      await del('dist')
      console.log('* removed useless /dist directory')
      server.close()
    })
    const generator = new Generator(nuxt, builder)
    await generator.generate()
  }
  else 
    app.use(nuxt.render)
}