const fs = require('fs-extra')
const path = require('path')
const Url = require('./url')
const moment = require('moment-timezone')
// TODO : make jsdoc
// TODO : cleaner code
class KiwiGenerator {
  constructor (configs) {
    this.configs = configs
  }
  // TODO: recursive -> Queue
  async exploreKiwi (url) {
    let { kiwiPath, isDebug } = this.configs
    let routes = []
    let childDocs = []
    let childDir = []
    let folderPath = path.join(kiwiPath, url.toString())

    if(isDebug) console.log(`* Hi. exploreKiwi at ${folderPath}`)
    // Read wiki Folder
    let files = await fs.readdir(folderPath)
    const target = '/kiwi'+url.toString()
    
    for(let file of files){
      const filePath = path.join(folderPath, file)
      let Stats = await fs.stat(filePath)

      const isIgnored = /^_(.*?)$/gm;

      // we accept only folder and .md      
      if(Stats.isFile() && path.extname(file) == '.md'){
        if(file.match(isIgnored)){
          if(isDebug) console.log(`* ${file} will be ignored.`)
          continue; // file starting with _ will be ignored
        } 
        else {
          if(isDebug) console.log(`* ${file} will be added. (file)`)
        }
        routes.push({
          route: '/kiwi'+new Url(url.join(file)).toString().replace('/index.md','').replace('.md',''),
          payload: {
            atime: Stats.atime, 
            birthtime: Stats.birthtime, 
            parent: '/kiwi'+url.toString(),
            path: filePath,
            isDir: file != 'index.md' ? false: true,
            isEmpty: false
          }
        })  
        if(file != 'index.md') // childDocs doesn't contain itself
          childDocs.push('/kiwi'+url.toString()+'/'+path.basename(file, path.extname(file)))
      }
      else if(Stats.isDirectory()){
        if(isDebug) console.log(`* ${file} will be added. (directory)`)
        let newUrl = new Url(url.join(file));
        childDir.push('/kiwi'+newUrl.toString())
        routes = routes.concat(await this.exploreKiwi(newUrl)) /// recursive for searching directory
      }
    }
    // If there is no index.md at this folder -> make blank one
    if(!files.includes('index.md')){
      routes.push({
        route: target,
        payload: {
          atime: null, 
          birthtime: null, 
          parent: '/kiwi'+new Url(url.cd()).toString(), 
          path: null,
          isDir: true,
          isEmpty: true
        }
      })  
    }
    
    if(isDebug){
      // hello from debug
      console.log(`* route results : `)
      console.log(routes)
    }
    // Add childDocs, childDir list to directory docs if they exist
    const index = routes.findIndex(x=>x.route == target)
    if(index != -1){
      if(childDocs.length > 0) {
        Object.assign(routes[index].payload, {childDocs})
      }
      if(childDir.length > 0) {
        Object.assign(routes[index].payload, {childDir})
      }
    }
    return routes
  }
  async generateBaseContents(map) {
    
    const { kiwiMapPath, kiwiBCPath, isDebug } = this.configs
    if(isDebug) console.log(`kiwiMapPath: ${kiwiMapPath}, kiwiBCPath : ${kiwiBCPath}`)
    if(!map){
      map = await fs.readFile(kiwiMapPath, 'utf8')
    }
    let updatedDocs = map.sort((b,a)=>moment(a.payload.atime).valueOf() - moment(b.payload.atime).valueOf()).map(x=>x.route) || []
    if(updatedDocs.length > 8)
    updatedDocs = updatedDocs.splice(0,8)

    let createdDocs = map.sort((b,a)=>moment(a.payload.birthtime).valueOf() - moment(b.payload.birthtime).valueOf()).map(x=>x.route) || []
    if(createdDocs.length > 8)
      createdDocs = createdDocs.splice(0,8)

    return {
      createdDocs, updatedDocs
    }
  }
  async generateMap () {
    return await this.exploreKiwi(new Url)
  }
  async generate (){
    const { kiwiMapPath, kiwiBCPath, isDebug } = this.configs
    try {
      const kiwiMap = await this.generateMap()

      await fs.writeFile(kiwiMapPath, JSON.stringify(kiwiMap, null, 2))
      console.log('* generating map, routes is done')    
  
      const baseContents = await this.generateBaseContents(kiwiMap)
      await fs.writeFile(kiwiBCPath, JSON.stringify(baseContents, null, 2))  
      console.log('* generating base-contents is done')
  
      return {
        kiwiMap, baseContents
      }  
    }
    catch(e){
      console.log('* generate(): failed');
      if(isDebug) console.log(e)
    }

  }
}
module.exports = {
  KiwiGenerator
}