const fs = require('fs-extra')
const moment = require('moment-timezone')
const hljs = require('highlight.js'); // https://highlightjs.org/
const md = require('markdown-it')({
  highlight: function (str, lang) {
    if (lang && hljs.getLanguage(lang)) {
      try {
        return '<pre class="hljs"><code>' +
               hljs.highlight(lang, str, true).value +
               '</code></pre>';
      } catch (__) {}
    }
    return '<pre class="hljs"><code>' + md.utils.escapeHtml(str) + '</code></pre>';
  },
  linkify: true
})
md.use(require('markdown-it-anchor'))
md.use(require('markdown-it-table-of-contents'))

module.exports = async (configs, payload) => {
  const { path, atime, birthtime, isEmpty } = payload
  console.log(`render request for ${path}`)
  let contents = ''
  try {
    if(!isEmpty){
      payload._atime = moment(atime).tz('Asia/Seoul').format('YYYY년 MM월 DD일 HH시 mm분 ss초')
      payload._birthtime = moment(birthtime).tz('Asia/Seoul').format('YYYY년 MM월 DD일 HH시 mm분 ss초')  
      contents = await fs.readFile(path, 'utf8')
      contents = md.render(contents)
    }  
  }
  catch (error) {
    contents = `Error occured during rendering markdown. <br> ${error}`
  }
  return {
    contents, payload
  }
}
